@@ -89,9 +89,6 @@ class ACSS(dict):
         """Update name if we delete a key."""
         dict.__delitem__(self, key)
 
-    def updateName(self):
-        """Update name based on settings."""
-
     def name(self):
         _name = 'acss-'
         names = list(self.keys())
@@ -105,3 +102,7 @@ class ACSS(dict):
     def getLocale(self):
         family = self.get(ACSS.FAMILY, {})
         return family.get('locale')
+
+    def getDialect(self):
+        family = self.get(ACSS.FAMILY, {})
+        return family.get('dialect')
