@@ -76,6 +76,10 @@ ROLE_HEADING_LEVEL_SPEECH = _("%(role)s level %(level)d")
 # of icons.
 ROLE_ICON_PANEL = _("Icon panel")
 
+# Translators: This is a state which applies to elements in document content
+# which have an "onClick" action.
+STATE_CLICKABLE = _("clickable")
+
 # Translators: This is a state which applies to items which can be expanded
 # or collapsed such as combo boxes and nodes/groups in a treeview. Collapsed
 # means the item's children are not showing; expanded means they are.
@@ -86,6 +90,10 @@ STATE_COLLAPSED = _("collapsed")
 # means the item's children are not showing; expanded means they are.
 STATE_EXPANDED = _("expanded")
 
+# Translators: This is a state which applies to elements in document conent
+# which have a longdesc attribute. http://www.w3.org/TR/WCAG20-TECHS/H45.html
+STATE_HAS_LONGDESC = _("has long description")
+
 # Translators: This is a state which applies to the orientation of widgets
 # such as sliders and scroll bars.
 STATE_HORIZONTAL = _("horizontal")
