@@ -34,6 +34,7 @@ import pyatspi
 
 import orca.messages as messages
 import orca.object_properties as object_properties
+import orca.orca_state as orca_state
 import orca.settings_manager as settings_manager
 import orca.speech_generator as speech_generator
 
@@ -54,6 +55,18 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
     def __init__(self, script):
         speech_generator.SpeechGenerator.__init__(self, script)
 
+    def _getACSS(self, obj, string):
+        if obj.getRole() == pyatspi.ROLE_LINK:
+            acss = self.voice(speech_generator.HYPERLINK)
+        elif isinstance(string, str) \
+            and string.isupper() \
+            and string.strip().isalpha():
+            acss = self.voice(speech_generator.UPPERCASE)
+        else:
+            acss = self.voice(speech_generator.DEFAULT)
+
+        return acss
+
     def _generateName(self, obj, **args):
         result = []
         acss = self.voice(speech_generator.DEFAULT)
@@ -90,7 +103,6 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                             break
                 if child and child.name:
                     result.append(child.name)
-                if result:
                     result.extend(acss)
 
         else:
@@ -98,6 +110,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                               self, obj, **args))
         if not result and role == pyatspi.ROLE_LIST_ITEM:
             result.append(self._script.utilities.expandEOCs(obj))
+            result.extend(acss)
 
         acss = self.voice(speech_generator.HYPERLINK)
         link = None
@@ -123,85 +136,56 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if that description is different from that of the name and
         label.
         """
-        if args.get('role', obj.getRole()) == pyatspi.ROLE_LINK \
-           and obj.parent.getRole() == pyatspi.ROLE_IMAGE:
-            result = self._generateName(obj, **args)
-            result.append(messages.IMAGE_MAP_LINK)
-        else:
-            result = speech_generator.SpeechGenerator.\
-                           _generateDescription(self, obj, **args)
-        return result
+        if not obj == orca_state.locusOfFocus:
+            return []
+
+        formatType = args.get('formatType')
+        role = args.get('role', obj.getRole())
+        if role == pyatspi.ROLE_TEXT and formatType != 'basicWhereAmI':
+            return []
+
+        return speech_generator.SpeechGenerator._generateDescription(
+            self, obj, **args)
 
     def _generateLabel(self, obj, **args):
         acss = self.voice(speech_generator.DEFAULT)
         result = speech_generator.SpeechGenerator._generateLabel(self,
                                                                  obj,
                                                                  **args)
-        role = args.get('role', obj.getRole())
-        # We'll attempt to infer the label under some circumstances.
-        #
-        if not len(result) \
-           and role in [pyatspi.ROLE_CHECK_BOX,
-                        pyatspi.ROLE_COMBO_BOX,
-                        pyatspi.ROLE_ENTRY,
-                        pyatspi.ROLE_LIST,
-                        pyatspi.ROLE_LIST_BOX,
-                        pyatspi.ROLE_PARAGRAPH,
-                        pyatspi.ROLE_PASSWORD_TEXT,
-                        pyatspi.ROLE_RADIO_BUTTON,
-                        pyatspi.ROLE_TEXT] \
-           and self._script.inDocumentContent():
 
-            # We're having to hack around yet another Mozilla bug:
-            # https://bugzilla.mozilla.org/show_bug.cgi?id=960241
-            focusedOnly = role not in [pyatspi.ROLE_LIST, pyatspi.ROLE_LIST_BOX]
+        if self._script.utilities.shouldInferLabelFor(obj):
+            start = args.get('startOffset')
+            if isinstance(start, int) and start > 0:
+                return []
 
-            label = self._script.labelInference.infer(obj, focusedOnly)
+            label, objects = self._script.labelInference.infer(obj, False)
             if label:
                 result.append(label)
+                result.extend(acss)
 
         # XUL combo boxes don't always have a label for/by
         # relationship.  But, they will make their names be
         # the string of the thing labelling them.
         #
+        role = args.get('role', obj.getRole())
         if not len(result) \
            and role == pyatspi.ROLE_COMBO_BOX \
            and not self._script.inDocumentContent():
             result.append(obj.name)
-
-        if result:
             result.extend(acss)
-        return result
 
-    def _generateLabelAndName(self, obj, **args):
-        result = []
-        role = args.get('role', obj.getRole())
-        # For radio buttons, the label is handled as a context and we
-        # assume we don't have to guess it.  If we need to guess it,
-        # we need to add it to utterances.
-        #
-        if role == pyatspi.ROLE_RADIO_BUTTON \
-           and self._script.utilities.displayedLabel(obj):
-            result.extend(
-                speech_generator.SpeechGenerator._generateName(
-                    self, obj, **args))
-        else:
-            result.extend(
-                speech_generator.SpeechGenerator._generateLabelAndName(
-                    self, obj, **args))
         return result
 
     def _generateLabelOrName(self, obj, **args):
-        result = []
-        if obj.parent.getRole() == pyatspi.ROLE_AUTOCOMPLETE:
-            # This is the main difference between this class and the default
-            # class - we'll give this thing a name here, and we'll make it
-            # be the name of the autocomplete.
-            #
-            result.extend(self._generateLabelOrName(obj.parent, **args))
-        else:
-            result.extend(speech_generator.SpeechGenerator._generateLabelOrName(
-                self, obj, **args))
+        if self._script.utilities.isTextBlockElement(obj):
+            return []
+
+        result = speech_generator.SpeechGenerator._generateLabelOrName(
+            self, obj, **args)
+
+        if not result and obj.parent.getRole() == pyatspi.ROLE_AUTOCOMPLETE:
+            result = self._generateLabelOrName(obj.parent, **args)
+
         return result
 
     def _generateRoleName(self, obj, **args):
@@ -210,6 +194,9 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         acss = self.voice(speech_generator.SYSTEM)
         role = args.get('role', obj.getRole())
         force = args.get('force', False)
+        start = args.get('startOffset')
+        if role == pyatspi.ROLE_ENTRY and isinstance(start, int) and start > 0:
+            return []
 
         # Saying "menu item" for a combo box can confuse users. Therefore,
         # speak the combo box role instead.  Also, only do it if the menu
@@ -228,6 +215,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                           pyatspi.ROLE_MENU_ITEM,
                           pyatspi.ROLE_PARAGRAPH,
                           pyatspi.ROLE_SECTION,
+                          pyatspi.ROLE_TEXT,
                           pyatspi.ROLE_UNKNOWN]
         else:
             # We never ever want to speak 'unknown'
@@ -239,35 +227,43 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             if args.get('formatType', 'unfocused') != 'basicWhereAmI':
                 doNotSpeak.append(pyatspi.ROLE_LIST_ITEM)
                 doNotSpeak.append(pyatspi.ROLE_LIST)
+            if args.get('startOffset') != None and args.get('endOffset') != None:
+                doNotSpeak.append(pyatspi.ROLE_DOCUMENT_FRAME)
+                doNotSpeak.append(pyatspi.ROLE_ALERT)
 
         if not (role in doNotSpeak):
             if role == pyatspi.ROLE_IMAGE:
+                result.append(self.getLocalizedRoleName(obj, role))
+                result.extend(acss)
                 link = self._script.utilities.ancestorWithRole(
                     obj, [pyatspi.ROLE_LINK], [pyatspi.ROLE_DOCUMENT_FRAME])
                 if link:
                     result.append(self.getLocalizedRoleName(link))
+                    result.extend(acss)
 
-            if role == pyatspi.ROLE_HEADING:
-                level = self._script.getHeadingLevel(obj)
+            elif role == pyatspi.ROLE_HEADING:
+                level = self._script.utilities.headingLevel(obj)
                 if level:
                     result.append(object_properties.ROLE_HEADING_LEVEL_SPEECH % {
                         'role': self.getLocalizedRoleName(obj, role),
                         'level': level})
+                    result.extend(acss)
                 else:
                     result.append(self.getLocalizedRoleName(obj, role))
+                    result.extend(acss)
+
+            elif role == pyatspi.ROLE_LINK:
+                if obj.parent.getRole() == pyatspi.ROLE_IMAGE:
+                    result.append(messages.IMAGE_MAP_LINK)
+                    result.extend(acss)
+                else:
+                    result.append(self.getLocalizedRoleName(obj, role))
+                    result.extend(acss)
+                    if obj.childCount and obj[0].getRole() == pyatspi.ROLE_IMAGE:
+                        result.append(self.getLocalizedRoleName(obj[0]))
+                        result.extend(acss)
             else:
                 result.append(self.getLocalizedRoleName(obj, role))
-
-            if result:
-                result.extend(acss)
-
-            if role == pyatspi.ROLE_LINK \
-               and obj.childCount and obj[0].getRole() == pyatspi.ROLE_IMAGE:
-                # If this is a link with a child which is an image, we
-                # want to indicate that.
-                #
-                acss = self.voice(speech_generator.HYPERLINK)
-                result.append(self.getLocalizedRoleName(obj[0]))
                 result.extend(acss)
 
         return result
@@ -275,7 +271,10 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
     def _generateExpandedEOCs(self, obj, **args):
         """Returns the expanded embedded object characters for an object."""
         result = []
-        text = self._script.utilities.expandEOCs(obj)
+
+        startOffset = args.get('startOffset', 0)
+        endOffset = args.get('endOffset', -1)
+        text = self._script.utilities.expandEOCs(obj, startOffset, endOffset)
         if text:
             result.append(text)
         return result
@@ -296,6 +295,17 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     self, obj, **args))
         return result
 
+    def _generateNewAncestors(self, obj, **args):
+        # TODO - JD: This is not the right way to do this, but we can fix
+        # that as part of the removal of formatting strings.
+        start = args.get('startOffset')
+        end = args.get('endOffset')
+        if start != None or end != None:
+            return []
+
+        return speech_generator.SpeechGenerator._generateNewAncestors(
+            self, obj, **args)
+
     def _generateAncestors(self, obj, **args):
         role = args.get('role', obj.getRole())
         if role == pyatspi.ROLE_LINK:
@@ -374,6 +384,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             result.extend(speech_generator.SpeechGenerator.\
                                            generateSpeech(self, obj, **args))
             self._restoreRole(oldRole, args)
+        elif self._script.utilities.isLink(obj):
+            oldRole = self._overrideRole(pyatspi.ROLE_LINK, args)
+            result.extend(speech_generator.SpeechGenerator.\
+                                           generateSpeech(self, obj, **args))
+            self._restoreRole(oldRole, args)
         else:
             result.extend(speech_generator.SpeechGenerator.\
                                            generateSpeech(self, obj, **args))
@@ -385,7 +400,17 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             if attribute.startswith(attributeName):
                 return attribute.split(":")[1]
 
+    def _generateNewNodeLevel(self, obj, **args):
+        if self._script.utilities.isTextBlockElement(obj):
+            return []
+
+        return speech_generator.SpeechGenerator._generateNewNodeLevel(
+            self, obj, **args)
+
     def _generatePositionInList(self, obj, **args):
+        if self._script.utilities.isTextBlockElement(obj):
+            return []
+
         position = self.getAttribute(obj, "posinset")
         total = self.getAttribute(obj, "setsize")
         if position and total and not obj.getRole() in \
@@ -416,3 +441,63 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         # is all kinds of broken. Until that can be sorted out, try to filter
         # out some of the noise....
         return []
+
+    def _generateAnyTextSelection(self, obj, **args):
+        if not obj == orca_state.locusOfFocus:
+            return []
+
+        return speech_generator.SpeechGenerator._generateAnyTextSelection(
+            self, obj, **args)
+
+    def _generateAllTextSelection(self, obj, **args):
+        if not obj == orca_state.locusOfFocus:
+            return []
+
+        return speech_generator.SpeechGenerator._generateAllTextSelection(
+            self, obj, **args)
+
+    def _generateSubstring(self, obj, **args):
+        start = args.get('startOffset')
+        end = args.get('endOffset')
+        if start == None or end == None:
+            return []
+
+        string = self._script.utilities.substring(obj, start, end)
+        string = self._script.utilities.adjustForRepeats(string)
+        if not string:
+            return []
+
+        if not self._script.utilities.isEntry(obj) \
+           and not self._script.utilities.isPasswordText(obj):
+            string = string.strip()
+
+        result = [string]
+        result.extend(self._getACSS(obj, string))
+        return result
+
+    # TODO - JD: While working on the Gecko rewrite, I found a metric crapton
+    # of text generation methods (including, but not limited to, these below).
+    # Are these really all needed? Seriously??
+    def _generateCurrentLineText(self, obj, **args):
+        result = self._generateSubstring(obj, **args)
+        if result:
+            return result
+
+        return speech_generator.SpeechGenerator._generateCurrentLineText(
+            self, obj, **args)
+
+    def _generateDisplayedText(self, obj, **args):
+        result = self._generateSubstring(obj, **args)
+        if result:
+            return result
+
+        return speech_generator.SpeechGenerator._generateDisplayedText(
+            self, obj, **args)
+
+    def _generateTextContent(self, obj, **args):
+        result = self._generateSubstring(obj, **args)
+        if result:
+            return result
+
+        return speech_generator.SpeechGenerator._generateTextContent(
+            self, obj, **args)
